@@ -128,9 +128,11 @@ def checkConfig(cfg, crontab = True):
     if crontab:
         test = 'Install crontab'
         announceTest()
+
         if not cfg.setupCron():
             failed()
             return False
+
         okay()
 
     return True
@@ -187,9 +189,11 @@ def terminalSize():
     """
     for fd in (sys.stdin, sys.stdout, sys.stderr):
         try:
-            import fcntl, termios, struct
+            import fcntl
+            import termios
+            import struct
             return [int(x) for x in struct.unpack('hh', fcntl.ioctl(fd, termios.TIOCGWINSZ, '1234'))]
-        except:
+        except ImportError:
             pass
     return [24, 80]
 
