@@ -1,21 +1,20 @@
-#    Back In Time
-#    Copyright (C) 2008-2022 Oprea Dan, Bart de Koning, Richard Bailey,
-#    Germar Reitze
+# Back In Time
+# Copyright (C) 2008-2022 Oprea Dan, Bart de Koning, Richard Bailey,
+# Germar Reitze
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 2 of the License, or
-#    (at your option) any later version.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
-#    You should have received a copy of the GNU General Public License along
-#    with this program; if not, write to the Free Software Foundation, Inc.,
-#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """Configuration logic.
 
 This module and its `Config` class contain the application logic handling the
@@ -28,19 +27,14 @@ Development notes:
     `backintime-config`. The script `create-manpage-backintime-config.py`
     parses this module for that.
 """
-
 import os
 import sys
 import datetime
 import socket
 import random
+import textwrap
+import getpass
 import shlex
-try:
-    import pwd
-except ImportError:
-    import getpass
-    pwd = None
-
 # Workaround: Mostly relevant on TravisCI but not exclusively.
 # While unittesting and without regular invocation of BIT the GNU gettext
 # class-based API isn't setup yet.
@@ -58,15 +52,16 @@ import sshtools
 import encfstools
 import password
 import pluginmanager
+import schedule
 from exceptions import PermissionDeniedByPolicy, \
                        InvalidChar, \
                        InvalidCmd, \
                        LimitExceeded
+import version
 
 
 class Config(configfile.ConfigFileWithProfiles):
     APP_NAME = 'Back In Time'
-    VERSION = '1.4.3'
     COPYRIGHT = 'Copyright (C) 2008-2024 Oprea Dan, Bart de Koning, ' \
                 'Richard Bailey, Germar Reitze, Christian Buhtz, Michael Büker, Jürgen Altfeld et al.'
 
@@ -140,16 +135,18 @@ class Config(configfile.ConfigFileWithProfiles):
     PLUGIN_MANAGER = pluginmanager.PluginManager()
 
     def __init__(self, config_path=None, data_path=None):
+        """Back In Time configuration (and much more then this).
+
+        Args:
+            config_path (str): Full path to the config file
+                (default: `~/.config/backintime/config`).
+            data_path (str): It is $XDG_DATA_HOME (default: `~/.local/share`).
+        """
         # Note: The main profiles name here is translated using the systems
         # current locale because the language code in the config file wasn't
         # read yet.
         configfile.ConfigFileWithProfiles.__init__(self, _('Main profile'))
 
-        self._APP_PATH = tools.backintimePath()
-        self._DOC_PATH = os.path.join(tools.sharePath(), 'doc', 'backintime-common')
-        if os.path.exists(os.path.join(self._APP_PATH, 'LICENSE')):
-            self._DOC_PATH = self._APP_PATH
-
         self._GLOBAL_CONFIG_PATH = '/etc/backintime/config'
 
         HOME_FOLDER = os.path.expanduser('~')
@@ -174,18 +171,22 @@ class Config(configfile.ConfigFileWithProfiles):
         tools.makeDirs(self._LOCAL_MOUNT_ROOT)
 
         self._DEFAULT_CONFIG_PATH = os.path.join(self._LOCAL_CONFIG_FOLDER, 'config')
+
         if config_path is None:
             self._LOCAL_CONFIG_PATH = self._DEFAULT_CONFIG_PATH
         else:
             self._LOCAL_CONFIG_PATH = os.path.abspath(config_path)
             self._LOCAL_CONFIG_FOLDER = os.path.dirname(self._LOCAL_CONFIG_PATH)
-        old_path = os.path.join(self._LOCAL_CONFIG_FOLDER, 'config2')
 
-        if os.path.exists(old_path):
-            if os.path.exists(self._LOCAL_CONFIG_PATH):
-                os.remove(old_path)
-            else:
-                os.rename(old_path, self._LOCAL_CONFIG_PATH)
+        # (buhtz) Introduced in 2009 via commit 5b26575be4.
+        # Ready to remove after 15 years.
+        # old_path = os.path.join(self._LOCAL_CONFIG_FOLDER, 'config2')
+
+        # if os.path.exists(old_path):
+        #     if os.path.exists(self._LOCAL_CONFIG_PATH):
+        #         os.remove(old_path)
+        #     else:
+        #         os.rename(old_path, self._LOCAL_CONFIG_PATH)
 
         # Load global config file
         self.load(self._GLOBAL_CONFIG_PATH)
@@ -209,7 +210,7 @@ class Config(configfile.ConfigFileWithProfiles):
                              "doesn't support upgrading config from version "   \
                              "< 1.0 anymore. Please use BackInTime version "    \
                              "<= 1.1.12 to upgrade the config to a more recent "\
-                             "version.".format(self.VERSION))
+                             "version.".format(version.__version__))
                 #TODO: add popup warning
                 sys.exit(2)
 
@@ -288,6 +289,15 @@ class Config(configfile.ConfigFileWithProfiles):
         # Workaround
         self.default_profile_name = _('Main profile')
 
+        # ToDo Those hidden labels exist to speed up their translation.
+        # Unhide them after the upcoming release (1.5.0).
+        # See: https://github.com/bit-team/backintime/issues/
+        # 1735#issuecomment-2197646518
+        _HIDDEN_NEW_MODE_LABELS = (
+            _('Local (EncFS encrypted)'),
+            _('SSH (EncFS encrypted)')
+        )
+
         self.SNAPSHOT_MODES = {
                     # mode: (
                     #     <mounttools>,
@@ -298,10 +308,10 @@ class Config(configfile.ConfigFileWithProfiles):
                     'local': (
                         None, _('Local'), False, False),
                     'ssh': (
-                        sshtools.SSH, 'SSH', _('SSH private key'), False),
+                        sshtools.SSH, _('SSH'), _('SSH private key'), False),
                     'local_encfs': (
                         encfstools.EncFS_mount,
-                        '{} {}'.format(_('Local'), _('encrypted')),
+                        _('Local encrypted'),
                         _('Encryption'),
                         False
                     ),
@@ -394,17 +404,6 @@ class Config(configfile.ConfigFileWithProfiles):
                         return False
         return True
 
-    def user(self):
-        """
-        portable way to get username
-        cc by-sa 3.0      http://stackoverflow.com/a/19865396/1139841
-        author: techtonik http://stackoverflow.com/users/239247/techtonik
-        """
-        if pwd:
-            return pwd.getpwuid(os.geteuid()).pw_name
-        else:
-            return getpass.getuser()
-
     def pid(self):
         return str(os.getpid())
 
@@ -500,7 +499,7 @@ class Config(configfile.ConfigFileWithProfiles):
         elif fs == 'fuse.sshfs' and mode not in ('ssh', 'ssh_encfs'):
             self.notifyError(_(
                 "Destination filesystem for {path} is an sshfs-mounted share."
-                " sshfs doesn't support hard-links. "
+                " Sshfs doesn't support hard-links. "
                 "Please use mode 'SSH' instead.")
                 .format(path=value))
 
@@ -626,7 +625,7 @@ class Config(configfile.ConfigFileWithProfiles):
 
     def sshUser(self, profile_id = None):
         #?Remote SSH user;;local users name
-        return self.profileStrValue('snapshots.ssh.user', self.user(), profile_id)
+        return self.profileStrValue('snapshots.ssh.user', getpass.getuser(), profile_id)
 
     def setSshUser(self, value, profile_id = None):
         self.setProfileStrValue('snapshots.ssh.user', value, profile_id)
@@ -662,6 +661,30 @@ class Config(configfile.ConfigFileWithProfiles):
     def setSshPrivateKeyFile(self, value, profile_id = None):
         self.setProfileStrValue('snapshots.ssh.private_key_file', value, profile_id)
 
+    def sshProxyHost(self, profile_id=None):
+        #?Proxy host used to connect to remote host.;;IP or domain address
+        return self.profileStrValue('snapshots.ssh.proxy_host', '', profile_id)
+
+    def setSshProxyHost(self, value, profile_id=None):
+        self.setProfileStrValue('snapshots.ssh.proxy_host', value, profile_id)
+
+    def sshProxyPort(self, profile_id=None):
+        #?Proxy host port used to connect to remote host.;0-65535
+        return self.profileIntValue(
+            'snapshots.ssh.proxy_host_port', '22', profile_id)
+
+    def setSshProxyPort(self, value, profile_id = None):
+        self.setProfileIntValue(
+            'snapshots.ssh.proxy_host_port', value, profile_id)
+
+    def sshProxyUser(self, profile_id=None):
+        #?Remote SSH user;;local users name
+        return self.profileStrValue(
+            'snapshots.ssh.proxy_user', getpass.getuser(), profile_id)
+
+    def setSshProxyUser(self, value, profile_id=None):
+        self.setProfileStrValue('snapshots.ssh.proxy_user', value, profile_id)
+
     def sshMaxArgLength(self, profile_id = None):
         #?Maximum command length of commands run on remote host. This can be tested
         #?for all ssh profiles in the configuration
@@ -707,16 +730,16 @@ class Config(configfile.ConfigFileWithProfiles):
         return args
 
     def sshCommand(self,
-                   cmd = None,
-                   custom_args = None,
-                   port = True,
-                   cipher = True,
-                   user_host = True,
-                   ionice = True,
-                   nice = True,
-                   quote = False,
-                   prefix = True,
-                   profile_id = None):
+                   cmd=None,
+                   custom_args=None,
+                   port=True,
+                   cipher=True,
+                   user_host=True,
+                   ionice=True,
+                   nice=True,
+                   quote=False,
+                   prefix=True,
+                   profile_id=None):
         """
         Return SSH command with all arguments.
 
@@ -735,20 +758,35 @@ class Config(configfile.ConfigFileWithProfiles):
         Returns:
             list:               ssh command with chosen arguments
         """
+        # Refactor: Use of assert is discouraged in productive code.
+        # Raise Exceptions instead.
         assert cmd is None or isinstance(cmd, list), "cmd '{}' is not list instance".format(cmd)
         assert custom_args is None or isinstance(custom_args, list), "custom_args '{}' is not list instance".format(custom_args)
-        ssh  = ['ssh']
+
+        ssh = ['ssh']
         ssh += self.sshDefaultArgs(profile_id)
+
+        # Proxy (aka Jump host)
+        if self.sshProxyHost(profile_id):
+            ssh += ['-J', '{}@{}:{}'.format(
+                self.sshProxyUser(profile_id),
+                self.sshProxyHost(profile_id),
+                self.sshProxyPort(profile_id)
+            )]
+
         # remote port
         if port:
             ssh += ['-p', str(self.sshPort(profile_id))]
+
         # cipher used to transfer data
         c = self.sshCipher(profile_id)
         if cipher and c != 'default':
-            ssh += ['-o', 'Ciphers={}'.format(c)]
+            ssh += ['-o', f'Ciphers={c}']
+
         # custom arguments
         if custom_args:
             ssh += custom_args
+
         # user@host
         if user_host:
             ssh.append('{}@{}'.format(self.sshUser(profile_id),
@@ -756,25 +794,32 @@ class Config(configfile.ConfigFileWithProfiles):
         # quote the command running on remote host
         if quote and cmd:
             ssh.append("'")
+
         # run 'ionice' on remote host
         if ionice and self.ioniceOnRemote(profile_id) and cmd:
             ssh += ['ionice', '-c2', '-n7']
+
         # run 'nice' on remote host
         if nice and self.niceOnRemote(profile_id) and cmd:
             ssh += ['nice', '-n19']
+
         # run prefix on remote host
         if prefix and cmd and self.sshPrefixEnabled(profile_id):
-            ssh += self.sshPrefixCmd(profile_id, cmd_type = list)
+            ssh += self.sshPrefixCmd(profile_id, cmd_type=type(cmd))
+
         # add the command
         if cmd:
             ssh += cmd
+
         # close quote
         if quote and cmd:
             ssh.append("'")
 
+        logger.debug(f'SSH command: {ssh}', self)
+
         return ssh
 
-    #ENCFS
+    # EncFS
     def localEncfsPath(self, profile_id = None):
         #?Where to save snapshots in mode 'local_encfs'.;absolute path
         return self.profileStrValue('snapshots.local_encfs.path', '', profile_id)
@@ -808,22 +853,35 @@ class Config(configfile.ConfigFileWithProfiles):
             mode = self.snapshotsMode(profile_id)
         self.setProfileBoolValue('snapshots.%s.password.use_cache' % mode, value, profile_id)
 
-    def password(self, parent = None, profile_id = None, mode = None, pw_id = 1, only_from_keyring = False):
-        if self.pw is None:
-            self.pw = password.Password(self)
-        if profile_id is None:
-            profile_id = self.currentProfile()
-        if mode is None:
-            mode = self.snapshotsMode(profile_id)
-        return self.pw.password(parent, profile_id, mode, pw_id, only_from_keyring)
+    def password(self,
+                 parent=None,
+                 profile_id=None,
+                 mode=None,
+                 pw_id=1,
+                 only_from_keyring=False):
 
-    def setPassword(self, password, profile_id = None, mode = None, pw_id = 1):
         if self.pw is None:
             self.pw = password.Password(self)
+
         if profile_id is None:
             profile_id = self.currentProfile()
+
         if mode is None:
             mode = self.snapshotsMode(profile_id)
+
+        return self.pw.password(
+            parent, profile_id, mode, pw_id, only_from_keyring)
+
+    def setPassword(self, password, profile_id=None, mode=None, pw_id=1):
+        if self.pw is None:
+            self.pw = password.Password(self)
+
+        if profile_id is None:
+            profile_id = self.currentProfile()
+
+        if mode is None:
+            mode = self.snapshotsMode(profile_id)
+
         self.pw.setPassword(password, profile_id, mode, pw_id)
 
     def modeNeedPassword(self, mode, pw_id = 1):
@@ -844,9 +902,9 @@ class Config(configfile.ConfigFileWithProfiles):
             profile_id = self.currentProfile()
         return 'profile_id_%s' % profile_id
 
-    def hostUserProfileDefault(self, profile_id = None):
+    def hostUserProfileDefault(self, profile_id=None):
         host = socket.gethostname()
-        user = self.user()
+        user = getpass.getuser()
         profile = profile_id
         if profile is None:
             profile = self.currentProfile()
@@ -952,6 +1010,13 @@ class Config(configfile.ConfigFileWithProfiles):
     def setScheduleMode(self, value, profile_id = None):
         self.setProfileIntValue('schedule.mode', value, profile_id)
 
+    def scheduleDebug(self, profile_id = None):
+        #?Enable debug output to system log for schedule mode.
+        return self.profileBoolValue('schedule.debug', False, profile_id)
+
+    def setScheduleDebug(self, value, profile_id = None):
+        self.setProfileBoolValue('schedule.debug', value, profile_id)
+
     def scheduleTime(self, profile_id = None):
         #?Position-coded number with the format "hhmm" to specify the hour
         #?and minute the cronjob should start (eg. 2015 means a quarter
@@ -1266,13 +1331,21 @@ class Config(configfile.ConfigFileWithProfiles):
         return self.setProfileBoolValue('snapshots.copy_unsafe_links', value, profile_id)
 
     def copyLinks(self, profile_id = None):
-        #?When  symlinks  are  encountered, the item that they point to
+        #?When symlinks are encountered, the item that they point to
         #?(the reference) is copied, rather than the symlink.
         return self.profileBoolValue('snapshots.copy_links', False, profile_id)
 
     def setCopyLinks(self, value, profile_id = None):
         return self.setProfileBoolValue('snapshots.copy_links', value, profile_id)
 
+    def oneFileSystem(self, profile_id = None):
+        #?Use rsync's "--one-file-system" to avoid crossing filesystem
+        #?boundaries when recursing.
+        return self.profileBoolValue('snapshots.one_file_system', False, profile_id)
+
+    def setOneFileSystem(self, value, profile_id = None):
+        return self.setProfileBoolValue('snapshots.one_file_system', value, profile_id)
+
     def rsyncOptionsEnabled(self, profile_id = None):
         #?Past additional options to rsync
         return self.profileBoolValue('snapshots.rsync_options.enabled', False, profile_id)
@@ -1312,17 +1385,25 @@ class Config(configfile.ConfigFileWithProfiles):
         self.setProfileBoolValue('snapshots.ssh.prefix.enabled', enabled, profile_id)
         self.setProfileStrValue('snapshots.ssh.prefix.value', value, profile_id)
 
-    def sshPrefixCmd(self, profile_id = None, cmd_type = str):
+    def sshPrefixCmd(self, profile_id=None, cmd_type=str):
+        """Return the config value of sshPrefix if enabled.
+
+        Dev note by buhtz (2024-04): Good opportunity to refactor. To much
+        implicit behavior in it.
+        """
         if cmd_type == list:
             if self.sshPrefixEnabled(profile_id):
                 return shlex.split(self.sshPrefix(profile_id))
-            else:
-                return []
+
+            return []
+
         if cmd_type == str:
             if self.sshPrefixEnabled(profile_id):
                 return self.sshPrefix(profile_id).strip() + ' '
-            else:
-                return ''
+
+            return ''
+
+        raise TypeError(f'Unable to handle type {cmd_type}.')
 
     def continueOnErrors(self, profile_id = None):
         #?Continue on errors. This will keep incomplete snapshots rather than
@@ -1367,30 +1448,29 @@ class Config(configfile.ConfigFileWithProfiles):
     def setGlobalFlock(self, value):
         self.setBoolValue('global.use_flock', value)
 
-    def appPath(self):
-        return self._APP_PATH
-
-    def docPath(self):
-        return self._DOC_PATH
-
     def appInstanceFile(self):
         return os.path.join(self._LOCAL_DATA_FOLDER, 'app.lock')
 
-    def fileId(self, profile_id = None):
+    def fileId(self, profile_id=None):
         if profile_id is None:
             profile_id = self.currentProfile()
+
         if profile_id == '1':
             return ''
+
         return profile_id
 
     def takeSnapshotLogFile(self, profile_id = None):
-        return os.path.join(self._LOCAL_DATA_FOLDER, "takesnapshot_%s.log" % self.fileId(profile_id))
+        return os.path.join(self._LOCAL_DATA_FOLDER,
+                            "takesnapshot_%s.log" % self.fileId(profile_id))
 
     def takeSnapshotMessageFile(self, profile_id = None):
-        return os.path.join(self._LOCAL_DATA_FOLDER, "worker%s.message" % self.fileId(profile_id))
+        return os.path.join(self._LOCAL_DATA_FOLDER,
+                            "worker%s.message" % self.fileId(profile_id))
 
     def takeSnapshotProgressFile(self, profile_id = None):
-        return os.path.join(self._LOCAL_DATA_FOLDER, "worker%s.progress" % self.fileId(profile_id))
+        return os.path.join(self._LOCAL_DATA_FOLDER,
+                            "worker%s.progress" % self.fileId(profile_id))
 
     def takeSnapshotInstanceFile(self, profile_id=None):
         return os.path.join(
@@ -1415,35 +1495,32 @@ class Config(configfile.ConfigFileWithProfiles):
     def cronEnvFile(self):
         return os.path.join(self._LOCAL_DATA_FOLDER, "cron_env")
 
-    def anacrontab(self, suffix = ''):
-        """
-        Deprecated since 1.1. Just keep this to delete old anacrontab files
-        """
-        return os.path.join(self._LOCAL_CONFIG_FOLDER, 'anacrontab' + suffix)
-
-    def anacrontabFiles(self):
-        """
-        list existing old anacrontab files
-        """
-        dirname, basename = os.path.split(self.anacrontab())
-        for f in os.listdir(dirname):
-            if f.startswith(basename):
-                yield os.path.join(dirname, f)
-
     def anacronSpool(self):
+        # ~/.local/share/backintime/anacron
         return os.path.join(self._LOCAL_DATA_FOLDER, 'anacron')
 
-    def anacronSpoolFile(self, profile_id = None):
-        return os.path.join(self.anacronSpool(), self.anacronJobIdentify(profile_id))
+    def anacronSpoolFile(self, profile_id=None):
+        """Return the timestamp file related to the current profile.
 
-    def anacronJobIdentify(self, profile_id = None):
+        Despite the methods name anacron is not involved. But the anacron
+        behavior is imitated by Back In Time. This timestamp files are an
+        element of this behavior.
+        """
+        # ~/.local/share/backintime/anacron/1_Main_profile
+        return os.path.join(self.anacronSpool(),
+                            self.anacronJobIdentify(profile_id))
+
+    def anacronJobIdentify(self, profile_id=None):
         if not profile_id:
             profile_id = self.currentProfile()
+
         profile_name = self.profileName(profile_id)
+
+        # "Main profile" -> "1_Main_profile"
         return profile_id + '_' + profile_name.replace(' ', '_')
 
     def udevRulesPath(self):
-        return os.path.join('/etc/udev/rules.d', '99-backintime-%s.rules' % self.user())
+        return os.path.join('/etc/udev/rules.d', '99-backintime-%s.rules' % getpass.getuser())
 
     def restoreLogFile(self, profile_id = None):
         return os.path.join(self._LOCAL_DATA_FOLDER, "restore_%s.log" % self.fileId(profile_id))
@@ -1459,23 +1536,6 @@ class Config(configfile.ConfigFileWithProfiles):
     def encfsconfigBackupFolder(self, profile_id = None):
         return os.path.join(self._LOCAL_DATA_FOLDER, 'encfsconfig_backup_%s' % self.fileId(profile_id))
 
-    def license(self):
-        return tools.readFile(os.path.join(self.docPath(), 'LICENSE'), '')
-
-    def translations(self):
-        return tools.readFile(os.path.join(self.docPath(), 'TRANSLATIONS'), '')
-
-    def authors(self):
-        return tools.readFile(os.path.join(self.docPath(), 'AUTHORS'), '')
-
-    def changelog(self):
-        for i in ('CHANGES', 'changelog'):
-            f = os.path.join(self.docPath(), i)
-            clog = tools.readFile(f, '')
-            if clog:
-                return clog
-        return ''
-
     def preparePath(self, path):
         if len(path) > 1:
             if path[-1] == os.sep:
@@ -1508,17 +1568,14 @@ class Config(configfile.ConfigFileWithProfiles):
         return True
 
     def backupScheduled(self, profile_id = None):
-        """
-        check if profile is supposed to be run this time
+        """Check if the profile is supposed to be run this time.
+
+        Returns:
+            (bool): The answer.
         """
         if self.scheduleMode(profile_id) not in (self.REPEATEDLY, self.UDEV):
             return True
 
-        #if crontab wasn't updated since upgrading BIT to version without anacron
-        #we are most likely started by anacron and should run this task without asking.
-        if list(self.anacrontabFiles()):
-            return True
-
         last_time = tools.readTimeStamp(self.anacronSpoolFile(profile_id))
         if not last_time:
             return True
@@ -1555,104 +1612,86 @@ class Config(configfile.ConfigFileWithProfiles):
         else:
             return True
 
-    SYSTEM_ENTRY_MESSAGE \
-        = "#Back In Time system entry, this will be edited by the gui:"
-    """The string is used in crontab file to mark entries as owned by Back
-    In Time. **WARNING**: Don't modify that string in code because it is used
-    as match target while parsing the crontab file.
-    """
-
     def setupCron(self):
-        for f in self.anacrontabFiles():
-            logger.debug("Clearing anacrontab %s"
-                         %f, self)
-            os.remove(f)
+        """Update the current users crontab file based on profile settings.
+
+        The crontab files is read, all entries related to Back In Time are
+        removed and after it added again for each profile based on the profile
+        settings. The difference between a backintime related entry created
+        by Back In Time itself or by the user manually is determined by a
+        comment before each entry. See :data:`schedule._MARKER` and
+        :func:`schedule.remove_bit_from_crontab()` for details.
+
+        Returns:
+            bool: ``True`` if successful or ``False`` on errors.
+        """
         self.setupUdev.clean()
 
-        oldCrontab = tools.readCrontab()
+        # Lines of current users crontab file
+        org_crontab_lines = schedule.read_crontab()
 
-        strippedCrontab = self.removeOldCrontab(oldCrontab)
-        newCrontab = self.createNewCrontab(strippedCrontab)
-        if not isinstance(newCrontab, (list, tuple)):
-            return newCrontab
+        # Remove all auto-generated BIT entries from crontab
+        crontab_lines = schedule.remove_bit_from_crontab(org_crontab_lines)
 
-        #save Udev rules
+        # Add a new entry to existing crontab content based on the current
+        # snapshot profile and its schedule settings.
+        crontab_lines = schedule.append_bit_to_crontab(
+            crontab_lines,
+            self.profiles_cron_lines())
+
+        # Save Udev rules
         try:
             if self.setupUdev.isReady and self.setupUdev.save():
                 logger.debug('Udev rules added successfully', self)
-        except PermissionDeniedByPolicy as e:
-            logger.error(str(e), self)
-            self.notifyError(str(e))
+
+        except PermissionDeniedByPolicy as err:
+            logger.error(str(err), self)
+            self.notifyError(str(err))
             return False
 
-        if not newCrontab == oldCrontab:
+        # Crontab modified?
+        if crontab_lines == org_crontab_lines:
+            return True
 
-            if not tools.checkCommand('crontab'):
+        if schedule.write_crontab(crontab_lines) == False:
+            logger.error('Failed to write new crontab.')
+            self.notifyError(_('Failed to write new crontab.'))
+            return False
 
-                if self.scheduleMode() is self.NONE:
-                    return True
-
-                else:
-                    logger.error('crontab not found.', self)
-                    self.notifyError(_(
-                        "Can't find crontab.\n"
-                        "Are you sure cron is installed?\n"
-                        "If not you should disable all automatic backups."))
-                    return False
-
-            if not tools.writeCrontab(newCrontab):
-                self.notifyError(_('Failed to write new crontab.'))
-                return False
-
-        else:
-            logger.debug("Crontab didn't change. Skip writing.")
+        if not schedule.is_cron_running():
+            logger.error(
+                'Cron is not running despite the crontab command being '
+                'available. Scheduled backup jobs will not run.')
+            self.notifyError(_(
+                'Cron is not running despite the crontab command being '
+                'available. Scheduled backup jobs will not run. '
+                'Cron might be installed but not enabled. Try the command '
+                '"systemctl enable cron" or consult the support channels of '
+                'your GNU Linux distribution.'))
 
         return True
 
-    def removeOldCrontab(self, crontab):
-        #We have to check if the self.SYSTEM_ENTRY_MESSAGE is in use,
-        #if not then the entries are most likely from Back In Time 0.9.26
-        #or earlier.
-        if not self.SYSTEM_ENTRY_MESSAGE in crontab:
-            #Then the system entry message has not yet been used in this crontab
-            #therefore we assume all entries are system entries and clear them all.
-            #This is the old behavior
-            logger.debug("Clearing all Back In Time entries", self)
-            return [x for x in crontab if not 'backintime' in x]
-        else:
-            #clear all line peers which have a SYSTEM_ENTRY_MESSAGE followed by
-            #one backintime command line
-            logger.debug("Clearing system Back In Time entries", self)
-            delLines = []
-            for i, line in enumerate(crontab):
-                if self.SYSTEM_ENTRY_MESSAGE in line and \
-                    len(crontab) > i + 1 and        \
-                    'backintime' in crontab[i + 1]:
-                        delLines.extend((i, i + 1))
-            return [line for i, line in enumerate(crontab) if i not in delLines]
+    def profiles_cron_lines(self):
+        """Return a list of crontab lines for each of the existing profiles.
 
-    def createNewCrontab(self, oldCrontab):
-        newCrontab = oldCrontab[:]
-        if not tools.checkCommand('backintime'):
-            logger.error("Command 'backintime' not found", self)
-            return newCrontab
-        for profile_id in self.profiles():
-            cronLine = self.cronLine(profile_id)
-            if not isinstance(cronLine, str):
-                return cronLine
-            if cronLine:
-                newCrontab.append(self.SYSTEM_ENTRY_MESSAGE)
-                newCrontab.append(cronLine.replace('{cmd}', self.cronCmd(profile_id)))
+        Return:
+            list: The list of crontab lines.
+        """
+        profile_ids = self.profiles()
 
-        if newCrontab == oldCrontab:
-            # Leave one self.SYSTEM_ENTRY_MESSAGE in to prevent deleting of manual
-            # entries if there is no automatic entry.
-            newCrontab.append(self.SYSTEM_ENTRY_MESSAGE)
-            newCrontab.append("#Please don't delete these two lines, or all custom backintime "
-                              "entries will be deleted next time you call the gui options!")
-        return newCrontab
+        # For each profile: cronline and the command (backintime)
+        cron_lines = [
+            self._cron_line(pid).replace('{cmd}', self._cron_cmd(pid))
+            for pid in profile_ids
+        ]
 
-    def cronLine(self, profile_id):
+        # Remove empty lines (profiles not scheduled)
+        cron_lines = list(filter(None, cron_lines))
+
+        return cron_lines
+
+    def _cron_line(self, profile_id):
+        """Create a crontab line based on the snapshot profiles settings."""
         cron_line = ''
         profile_name = self.profileName(profile_id)
         backup_mode = self.scheduleMode(profile_id)
@@ -1737,7 +1776,7 @@ class Config(configfile.ConfigFileWithProfiles):
                 #cache uuid in config
                 self.setProfileStrValue('snapshots.path.uuid', uuid, profile_id)
             try:
-                self.setupUdev.addRule(self.cronCmd(profile_id), uuid)
+                self.setupUdev.addRule(self._cron_cmd(profile_id), uuid)
             except (InvalidChar, InvalidCmd, LimitExceeded) as e:
                 logger.error(str(e), self)
                 self.notifyError(str(e))
@@ -1751,29 +1790,55 @@ class Config(configfile.ConfigFileWithProfiles):
 
         return cron_line
 
-    def cronCmd(self, profile_id):
-        if not tools.checkCommand('backintime'):
-            logger.error("Command 'backintime' not found", self)
-            return
+    def _cron_cmd(self, profile_id):
+        """Generates the command used in the crontab file based on the settings
+        for the current profile.
+
+        Returns:
+            str: The crontab line.
+        """
+
+        # Get full path of the Back In Time binary
         cmd = tools.which('backintime') + ' '
+
+        # The "--profile-id" argument is used only for profiles different from
+        # first profile
         if profile_id != '1':
             cmd += '--profile-id %s ' % profile_id
+
+        # User defined path to config file
         if not self._LOCAL_CONFIG_PATH is self._DEFAULT_CONFIG_PATH:
             cmd += '--config %s ' % self._LOCAL_CONFIG_PATH
-        if logger.DEBUG:
+
+        # Enable debug output
+        if self.scheduleDebug(profile_id):
             cmd += '--debug '
+
+        # command
         cmd += 'backup-job'
+
+        # Redirect stdout to nirvana
         if self.redirectStdoutInCron(profile_id):
             cmd += ' >/dev/null'
+
+        # Redirect stderr ...
         if self.redirectStderrInCron(profile_id):
+
             if self.redirectStdoutInCron(profile_id):
+                # ... to stdout
                 cmd += ' 2>&1'
             else:
+                # ... to nirvana
                 cmd += ' 2>/dev/null'
+
+        # IO priority: low (-n7) in "best effort" class (-c2)
         if self.ioniceOnCron(profile_id) and tools.checkCommand('ionice'):
             cmd = tools.which('ionice') + ' -c2 -n7 ' + cmd
+
+        # CPU priority: very low
         if self.niceOnCron(profile_id) and tools.checkCommand('nice'):
             cmd = tools.which('nice') + ' -n19 ' + cmd
+
         return cmd
 
 
