@@ -107,6 +107,7 @@
 import os
 import subprocess
 import json
+import getpass
 from zlib import crc32
 from time import sleep
 
@@ -450,7 +451,7 @@ class MountControl(object):
         self.symlink = symlink
 
         self.local_host = self.config.host()
-        self.local_user = self.config.user()
+        self.local_user = getpass.getuser()
         self.pid = self.config.pid()
 
         self.all_kwargs = {}
@@ -603,11 +604,11 @@ class MountControl(object):
         try:
             subprocess.check_call(['fusermount', '-u', self.currentMountpoint])
 
-        except subprocess.CalledProcessError:
+        except subprocess.CalledProcessError as exc:
             raise MountException(
                 _("Can't unmount {mountprocess} from {mountpoint}.")
                 .format(mountprocess=self.mountproc,
-                        mountpoint=self.currentMountpoint))
+                        mountpoint=self.currentMountpoint)) from exc
 
     def preMountCheck(self, first_run=False):
         """
@@ -692,9 +693,11 @@ class MountControl(object):
             logger.debug(f'{self.mountproc} is missing', self)
 
             raise MountException(
-                _('{} not found. Please install e.g. {}')
-                .format(self.mountproc,
-                        f"'apt-get install {self.mountproc}'"))
+                _('{command} not found. Please install it '
+                  '(e.g. via "{installcommand}")').format(
+                      command=self.mountproc,
+                      installcommand=f"'apt-get install {self.mountproc}'")
+            )
 
     def mounted(self):
         """
@@ -713,8 +716,9 @@ class MountControl(object):
         else:
             try:
                 if os.listdir(self.currentMountpoint):
-                    raise MountException(_('Mountpoint {} not empty.')
-                                         .format(self.currentMountpoint))
+                    raise MountException(
+                        _('Mountpoint {mntpoint} not empty.').format(
+                            mntpoint=self.currentMountpoint))
 
             except FileNotFoundError:
                 pass
@@ -1035,7 +1039,7 @@ class MountControl(object):
         Returns:
             str:        hash of string ``s``
         """
-        return('%X' % (crc32(s.encode()) & 0xFFFFFFFF))
+        return '%X' % (crc32(s.encode()) & 0xFFFFFFFF)
 
     def hashIdPath(self, hash_id = None):
         """
