@@ -1,20 +1,20 @@
-#    Back In Time
-#    Copyright (C) 2008-2022 Oprea Dan, Bart de Koning, Richard Bailey, Germar Reitze
+#  Back In Time
+#  Copyright (C) 2008-2022 Oprea Dan, Bart de Koning, Richard Bailey,
+#  Germar Reitze
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 2 of the License, or
-#    (at your option) any later version.
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
 #
-#    You should have received a copy of the GNU General Public License along
-#    with this program; if not, write to the Free Software Foundation, Inc.,
-#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-
+#  You should have received a copy of the GNU General Public License along
+#  with this program; if not, write to the Free Software Foundation, Inc.,
+#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 import os
 import pluginmanager
@@ -23,7 +23,7 @@ import gettext
 from subprocess import Popen, PIPE
 from exceptions import StopException
 
-_=gettext.gettext
+_ = gettext.gettext
 
 
 class UserCallbackPlugin(pluginmanager.Plugin):
@@ -33,14 +33,11 @@ class UserCallbackPlugin(pluginmanager.Plugin):
     files) about different steps ("events") in the backup process
     via the :py:class:`pluginmanager.PluginManager`.
 
-    This plugin calls a user-defined script file ("user-callback")
-    that is located in this folder:
+    This plugin calls a user-defined script file ("user-callback"). By default
+    that file is located in the config folder `$XDG_CONFIG_HOME/backintime/`
+    or another folder if command line optioni `--config` is used.
 
-        $XDG_CONFIG_HOME/backintime/user-callback
-        (by default $XDG_CONFIG_HOME is ~/.config)
-
-    The user-callback script is called with up to five
-    positional arguments:
+    The user-callback script is called with up to five positional arguments:
 
     1. The profile ID from the config file (1=Main Profile, ...)
     2. The profile name (from the config file)
@@ -67,37 +64,53 @@ class UserCallbackPlugin(pluginmanager.Plugin):
     def init(self, snapshots):
         self.config = snapshots.config
         self.script = self.config.takeSnapshotUserCallback()
-        if not os.path.exists(self.script):
-            return False
-        return True
 
-    # TODO 09/28/2022: This method should be private (__callback)
-    def callback(self, *args, profileID = None):
+        return os.path.exists(self.script)
+
+    # TODO 09/28/2022: This method should be private (_callback)
+    def callback(self, *args, profileID=None):
         if profileID is None:
             profileID = self.config.currentProfile()
+
         profileName = self.config.profileName(profileID)
         cmd = [self.script, profileID, profileName]
-        cmd.extend([str(x) for x in args])
-        logger.debug('Call user-callback: %s' %' '.join(cmd), self)
+        cmd.extend(str(x) for x in args)
+
+        logger.debug(f'Call user-callback: {" ".join(cmd)}', self)
+
         if self.config.userCallbackNoLogging():
             stdout, stderr = None, None
         else:
             stdout, stderr = PIPE, PIPE
+
         try:
             callback = Popen(cmd,
-                             stdout = stdout,
-                             stderr = stderr,
-                             universal_newlines = True)
+                             stdout=stdout,
+                             stderr=stderr,
+                             universal_newlines=True)
             output = callback.communicate()
+
+            # Stdout
             if output[0]:
-                logger.info('user-callback returned \'%s\'' %output[0].strip('\n'), self)
+                logger.info("user-callback returned '"
+                            + output[0].strip('\n') + "'",
+                            self)
+
+            # Stderr
             if output[1]:
-                logger.error('user-callback returned \'%s\'' %output[1].strip('\n'), self)
+                logger.error("user-callback returned '"
+                             + output[1].strip('\n') + "'",
+                             self)
+
             if callback.returncode != 0:
-                logger.warning('user-callback returncode: %s' %callback.returncode, self)
+                logger.warning(
+                    f'user-callback returncode: {callback.returncode}', self)
                 raise StopException()
+
         except OSError as e:
-            logger.error("Exception when trying to run user callback: %s" % e.strerror, self)
+            logger.error(
+                f'Exception when trying to run user callback: {e.strerror}',
+                self)
 
     def processBegin(self):
         self.callback('1')
