@@ -15,7 +15,6 @@
 # You should have received a copy of the GNU General Public License along
 # with this program; if not, write to the Free Software Foundation,Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-
 import os
 import sys
 import subprocess
@@ -26,17 +25,15 @@ import stat
 import signal
 import unittest
 from unittest.mock import patch
-import uuid
 from copy import deepcopy
 from tempfile import NamedTemporaryFile, TemporaryDirectory
 from datetime import datetime
 from test import generic
 from time import sleep
-from pyfakefs.fake_filesystem_unittest import patchfs
+import pyfakefs.fake_filesystem_unittest as pyfakefs_ut
 
 sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
 import tools
-import config
 import configfile
 
 # chroot jails used for building may have no UUID devices (because of tmpfs)
@@ -88,9 +85,9 @@ class TestTools(generic.TestCase):
 
     def tearDown(self):
         super(TestTools, self).tearDown()
-        self.killProcess()
+        self._kill_process()
 
-    def createProcess(self, *args):
+    def _create_process(self, *args):
         dummyPath = os.path.join(os.path.dirname(__file__), generic.DUMMY)
         cmd = [dummyPath]
         cmd.extend(args)
@@ -98,7 +95,7 @@ class TestTools(generic.TestCase):
         sleep(0.1)
         return self.subproc.pid
 
-    def killProcess(self):
+    def _kill_process(self):
         if self.subproc:
             self.subproc.kill()
             self.subproc.wait()
@@ -129,18 +126,6 @@ class TestTools(generic.TestCase):
         tools.addSourceToPathEnviron()
         self.assertIn(source, os.environ['PATH'])
 
-    def test_gitRevisionAndHash(self):
-        ref, hashid = tools.gitRevisionAndHash()
-        if isinstance(ref, str):
-            self.assertGreater(len(ref), 0)
-        else:
-            self.assertIsNone(ref)
-
-        if isinstance(hashid, str):
-            self.assertEqual(len(hashid), 7)
-        else:
-            self.assertIsNone(hashid)
-
     def test_readFile(self):
         """
         Test the function readFile
@@ -246,7 +231,7 @@ class TestTools(generic.TestCase):
         self.assertIn(os.getpid(), pids)
 
     def test_processStat(self):
-        pid = self.createProcess()
+        pid = self._create_process()
         stat = tools.processStat(pid)
         self.assertRegex(
             stat, r'{} \({}\) \w .*'.format(pid, generic.DUMMY[:15]))
@@ -254,11 +239,11 @@ class TestTools(generic.TestCase):
     @patch('builtins.open')
     def test_processStat_exception(self, mock_open):
         mock_open.side_effect = OSError()
-        pid = self.createProcess()
+        pid = self._create_process()
         self.assertEqual(tools.processStat(pid), '')
 
     def test_processPaused(self):
-        pid = self.createProcess()
+        pid = self._create_process()
         self.assertFalse(tools.processPaused(pid))
         self.subproc.send_signal(signal.SIGSTOP)
         sleep(0.01)
@@ -268,34 +253,34 @@ class TestTools(generic.TestCase):
         self.assertFalse(tools.processPaused(pid))
 
     def test_processName(self):
-        pid = self.createProcess()
+        pid = self._create_process()
         self.assertEqual(tools.processName(pid), generic.DUMMY[:15])
 
     def test_processCmdline(self):
-        pid = self.createProcess()
+        pid = self._create_process()
         self.assertRegex(tools.processCmdline(pid),
                          r'.*/sh.*/common/test/dummy_test_process\.sh')
-        self.killProcess()
-        pid = self.createProcess('foo', 'bar')
+        self._kill_process()
+        pid = self._create_process('foo', 'bar')
         self.assertRegex(tools.processCmdline(pid),
                          r'.*/sh.*/common/test/dummy_test_process\.sh.foo.bar')
 
     @patch('builtins.open')
     def test_processCmdline_exception(self, mock_open):
         mock_open.side_effect = OSError()
-        pid = self.createProcess()
+        pid = self._create_process()
         self.assertEqual(tools.processCmdline(pid), '')
 
     def test_pidsWithName(self):
         self.assertEqual(len(tools.pidsWithName('nonExistingProcess')), 0)
-        pid = self.createProcess()
+        pid = self._create_process()
         pids = tools.pidsWithName(generic.DUMMY)
         self.assertGreaterEqual(len(pids), 1)
         self.assertIn(pid, pids)
 
     def test_processExists(self):
         self.assertFalse(tools.processExists('nonExistingProcess'))
-        self.createProcess()
+        self._create_process()
         self.assertTrue(tools.processExists(generic.DUMMY))
 
     def test_processAlive(self):
@@ -304,9 +289,9 @@ class TestTools(generic.TestCase):
         """
         # TODO: add test (in chroot) running proc as root and kill as non-root
         self.assertTrue(tools.processAlive(os.getpid()))
-        pid = self.createProcess()
+        pid = self._create_process()
         self.assertTrue(tools.processAlive(pid))
-        self.killProcess()
+        self._kill_process()
         self.assertFalse(tools.processAlive(pid))
         self.assertFalse(tools.processAlive(999999))
         with self.assertRaises(ValueError):
@@ -406,8 +391,6 @@ class TestTools(generic.TestCase):
         self.assertFalse(tools.checkCronPattern('*/6,8'))
         self.assertFalse(tools.checkCronPattern('*/6 a'))
 
-    # envLoad and envSave tests are in TestToolsEnviron below
-
     def test_mountpoint(self):
         self.assertEqual(tools.mountpoint('/nonExistingFolder/foo/bar'), '/')
         proc = os.path.join('/proc', str(os.getpid()), 'fd')
@@ -432,215 +415,6 @@ class TestTools(generic.TestCase):
         self.assertEqual(procArgs[1], '/proc')
         self.assertEqual(procArgs[2], 'proc')
 
-    def test_device(self):
-        self.assertEqual(tools.device('/proc'), 'proc')
-        self.assertRegex(tools.device('/sys'), r'sys.*')
-        self.assertRegex(
-            tools.device('/nonExistingFolder/foo/bar'),
-            r'(:?/dev/.*|tmpfs|overlay|instances/containers/travis.*)')
-
-    def test_filesystem(self):
-        self.assertEqual(tools.filesystem('/proc'), 'proc')
-        self.assertRegex(tools.filesystem('/sys'), r'sys.*')
-        self.assertRegex(
-            tools.filesystem('/nonExistingFolder/foo/bar').lower(),
-            r'(:?ext[2-4]|xfs|zfs|jfs|raiserfs|btrfs|tmpfs|overlay|shiftfs)')
-
-    # tools.uuidFromDev() get called from tools.uuidFromPath because the
-    # latter is a synonym/surrogate for too.suuidFromDev()
-    # So we skip an extra unittest as it's hard to find a dev on all systems
-    @unittest.skipIf(not DISK_BY_UUID_AVAILABLE and not UDEVADM_HAS_UUID,
-                     'No UUIDs available on this system.')
-    def test_uuidFromPath(self):
-        """UUID related to a path.
-
-        by buhtz: I was wondering why this test passed because the path used
-        here doesn't exists! The function ``tools.uuidFromPath()`` does use
-        ``tools.device()`` function to determine the device (e.g. ``/sda1``)
-        related to the path. Not matter that the path itself doesn't exists
-        only the "mountpoint" is relevant. The mountpoint for the non-existing
-        path is ``/`` which of curse does exists.
-
-        In short: That test needs a refactoring.
-        """
-
-        uuid = tools.uuidFromPath('/nonExistingFolder/foo/bar')
-
-        self.assertIsInstance(uuid, str)
-        self.assertRegex(uuid.lower(), r'^[a-f0-9\-]+$')
-        self.assertEqual(len(uuid.replace('-', '')), 32)
-
-    @patchfs
-    def test_uuid_via_filesystem(self, fake_fs):
-        """Extract UUID from /dev filesystem.
-
-        That test using a faked filesystem via pyfakefs. 16 devices and
-        corresponding uuids are generated."""
-
-        # dev-disk folder
-        path_dev = pathlib.Path('/dev')
-        fake_fs.create_dir(path_dev)
-
-        # create disk-files from "sda1" to "sda4" to "sdd4"
-        dev_list = [
-            path_dev / f'sd{letter}{number}'
-            for letter in list('abcd')
-            for number in range(1, 5)]
-
-        # dev-disk-by-uuid
-        path_by_uuid = pathlib.Path('/dev') / 'disk' / 'by-uuid'
-        fake_fs.create_dir(path_by_uuid)
-
-        # uuids
-        uuid_list = [str(uuid.uuid4()) for _ in range(16)]
-
-        # connect device files with uuid symlinks
-        for idx in range(16):  # 16 devices
-            fake_fs.create_symlink(
-                # e.g. /dev/disk/by-uuid/c7aca0
-                file_path=path_by_uuid / uuid_list[idx],
-                # e.g. /dev/sda1
-                link_target=path_dev / dev_list[idx]
-            )
-
-        # randomly select one of the dev-uuid pairs
-        check_idx = random.choice(range(16))
-
-        # TEST
-        self.assertEqual(
-            tools._uuidFromDev_via_filesystem(dev=dev_list[check_idx]),
-            uuid_list[check_idx]
-        )
-
-    @patch('subprocess.check_output')
-    def test_uuid_via_blkid(self, mock_check_output):
-        """Extract UUID from blkid output."""
-
-        one_uuid = 'c7aca0a7-89ed-43f0-a4f9-c744dfe673e0'
-        one_dev = '/dev/sda1'
-
-        output = f'{one_dev}: UUID="{one_uuid}" BLOCK_SIZE="4096" ' \
-                 'TYPE="ext4" PARTUUID="89ffeb8f-01"'
-
-        mock_check_output.return_value = output
-
-        self.assertEqual(
-            tools._uuidFromDev_via_blkid_command(dev=one_dev),
-            one_uuid
-        )
-
-    @patch('subprocess.check_output')
-    def test_uuid_via_udevadm(self, mock_check_output):
-        """Extract UUID from udevadm output."""
-
-        one_uuid = 'c7aca0a7-89ed-43f0-a4f9-c744dfe673e0'
-        one_dev = '/dev/sda1'
-
-        # output to mock with injected dev and uuid
-        output = 'P: /devices/pci0000:00/0000:00:1f.2/ata1/host0/target' \
-                 '0:0:0/0:0:0:0/block/sda/sda1\n' \
-                 f'N: {one_dev}\n' \
-                 'L: 0\n' \
-                 'S: disk/by-uuid/c7aca0a7-89ed-43f0-a4f9-c744dfe673e0\n' \
-                 'S: disk/by-id/ata-WDC_WD20EARS-00S8B1_WD-' \
-                 'WCAVY4333133-part1\n' \
-                 'S: disk/by-id/wwn-0x50014ee2049ff22c-part1\n' \
-                 'S: disk/by-path/pci-0000:00:1f.2-ata-1.0-part1\n' \
-                 'S: disk/by-partuuid/89ffeb8f-01\n' \
-                 'S: disk/by-path/pci-0000:00:1f.2-ata-1-part1\n' \
-                 'E: DEVPATH=/devices/pci0000:00/0000:00:1f.2/ata1/host0/' \
-                 'target0:0:0/0:0:0:0/block/sda/sda1\n' \
-                 'E: DEVNAME=/dev/sda1\n' \
-                 'E: DEVTYPE=partition\n' \
-                 'E: PARTN=1\n' \
-                 'E: MAJOR=8\n' \
-                 'E: MINOR=1\n' \
-                 'E: SUBSYSTEM=block\n' \
-                 'E: USEC_INITIALIZED=1997408\n' \
-                 'E: ID_ATA=1\n' \
-                 'E: ID_TYPE=disk\n' \
-                 'E: ID_BUS=ata\n' \
-                 'E: ID_MODEL=WDC_WD20EARS-00S8B1\n' \
-                 'E: ID_MODEL_ENC=WDC\x20WD20EARS-00S8B1' \
-                 '\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20' \
-                 '\x20\x20\x20\x20\x20\x20\x20\x20\n' \
-                 'E: ID_REVISION=80.00A80\n' \
-                 'E: ID_SERIAL=WDC_WD20EARS-00S8B1_WD-WCAVY4333133\n' \
-                 'E: ID_SERIAL_SHORT=WD-WCAVY4333133\n' \
-                 'E: ID_ATA_WRITE_CACHE=1\n' \
-                 'E: ID_ATA_WRITE_CACHE_ENABLED=1\n' \
-                 'E: ID_ATA_FEATURE_SET_HPA=1\n' \
-                 'E: ID_ATA_FEATURE_SET_HPA_ENABLED=1\n' \
-                 'E: ID_ATA_FEATURE_SET_PM=1\n' \
-                 'E: ID_ATA_FEATURE_SET_PM_ENABLED=1\n' \
-                 'E: ID_ATA_FEATURE_SET_SECURITY=1\n' \
-                 'E: ID_ATA_FEATURE_SET_SECURITY_ENABLED=0\n' \
-                 'E: ID_ATA_FEATURE_SET_SECURITY_ERASE_UNIT_MIN=416\n' \
-                 'E: ID_ATA_FEATURE_SET_SECURITY_ENHANCED_' \
-                 'ERASE_UNIT_MIN=416\n' \
-                 'E: ID_ATA_FEATURE_SET_SECURITY_FROZEN=1\n' \
-                 'E: ID_ATA_FEATURE_SET_SMART=1\n' \
-                 'E: ID_ATA_FEATURE_SET_SMART_ENABLED=1\n' \
-                 'E: ID_ATA_FEATURE_SET_AAM=1\n' \
-                 'E: ID_ATA_FEATURE_SET_AAM_ENABLED=1\n' \
-                 'E: ID_ATA_FEATURE_SET_AAM_VENDOR_RECOMMENDED_VALUE=128\n' \
-                 'E: ID_ATA_FEATURE_SET_AAM_CURRENT_VALUE=254\n' \
-                 'E: ID_ATA_FEATURE_SET_PUIS=1\n' \
-                 'E: ID_ATA_FEATURE_SET_PUIS_ENABLED=0\n' \
-                 'E: ID_ATA_DOWNLOAD_MICROCODE=1\n' \
-                 'E: ID_ATA_SATA=1\n' \
-                 'E: ID_ATA_SATA_SIGNAL_RATE_GEN2=1\n' \
-                 'E: ID_ATA_SATA_SIGNAL_RATE_GEN1=1\n' \
-                 'E: ID_WWN=0x50014ee2049ff22c\n' \
-                 'E: ID_WWN_WITH_EXTENSION=0x50014ee2049ff22c\n' \
-                 'E: ID_PATH=pci-0000:00:1f.2-ata-1.0\n' \
-                 'E: ID_PATH_TAG=pci-0000_00_1f_2-ata-1_0\n' \
-                 'E: ID_PATH_ATA_COMPAT=pci-0000:00:1f.2-ata-1\n' \
-                 'E: ID_PART_TABLE_UUID=89ffeb8f\n' \
-                 'E: ID_PART_TABLE_TYPE=dos\n' \
-                 'E: ID_FS_UUID=c7aca0a7-89ed-43f0-a4f9-c744dfe673e0\n' \
-                 'E: ID_FS_UUID_ENC=c7aca0a7-89ed-43f0-a4f9-c744dfe673e0\n' \
-                 'E: ID_FS_VERSION=1.0\n' \
-                 'E: ID_FS_TYPE=ext4\n' \
-                 'E: ID_FS_USAGE=filesystem\n' \
-                 'E: ID_PART_ENTRY_SCHEME=dos\n' \
-                 'E: ID_PART_ENTRY_UUID=89ffeb8f-01\n' \
-                 'E: ID_PART_ENTRY_TYPE=0x83\n' \
-                 'E: ID_PART_ENTRY_NUMBER=1\n' \
-                 'E: ID_PART_ENTRY_OFFSET=2048\n' \
-                 'E: ID_PART_ENTRY_SIZE=3907026944\n' \
-                 'E: ID_PART_ENTRY_DISK=8:0\n' \
-                 'E: DEVLINKS=/dev/disk/by-uuid/c7aca0a7-89ed-43f0-a4f9-' \
-                 'c744dfe673e0 /dev/disk/by-id/ata-WDC_WD20EARS-00S8B1_WD'\
-                 '-WCAVY4333133-part1 /dev/disk/by-id/wwn-' \
-                 '0x50014ee2049ff22c-part1 /dev/disk/by-path/pci-0000:00:' \
-                 '1f.2-ata-1.0-part1 /dev/disk/by-partuuid/89ffeb8f-01 ' \
-                 '/dev/disk/by-path/pci-0000:00:1f.2-ata-1-part1\n' \
-                 'E: TAGS=:systemd:\n' \
-                 'E: CURRENT_TAGS=:systemd:\n'
-
-        mock_check_output.return_value = output
-
-        self.assertEqual(
-            tools._uuidFromDev_via_udevadm_command(dev=one_dev),
-            one_uuid
-        )
-
-    @unittest.skipIf(not DISK_BY_UUID_AVAILABLE and not UDEVADM_HAS_UUID,
-                     'No UUIDs available on this system.')
-    def test_filesystemMountInfo(self):
-        """
-        Basic sanity checks on returned structure
-        """
-        mounts = tools.filesystemMountInfo()
-        self.assertIsInstance(mounts, dict)
-        self.assertGreater(len(mounts.items()), 0)
-        self.assertIn('/', mounts)
-        self.assertIn('original_uuid', mounts.get('/'))
-
-    def test_syncfs(self):
-        self.assertTrue(tools.syncfs())
-
     def test_isRoot(self):
         self.assertIsInstance(tools.isRoot(), bool)
 
@@ -692,25 +466,6 @@ class TestTools(generic.TestCase):
             self.assertTrue(s.replace(' ', '').isdigit())
             self.assertEqual(len(s), 13)
 
-    @unittest.skipIf(not tools.checkCommand('crontab'),
-                     "'crontab' not found.")
-    def test_readWriteCrontab(self):
-        now = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
-        oldCrontab = tools.readCrontab()
-        self.assertIsInstance(oldCrontab, list)
-
-        testLine = '#BackInTime Unittest from {}. Test probably failed. ' \
-                   'You can remove this line.'.format(now)
-        self.assertTrue(tools.writeCrontab(oldCrontab + [testLine, ]))
-
-        newCrontab = tools.readCrontab()
-        self.assertIn(testLine, newCrontab)
-        self.assertEqual(len(newCrontab), len(oldCrontab) + 1)
-
-        self.assertTrue(tools.writeCrontab(oldCrontab))
-        if oldCrontab:
-            self.assertListEqual(oldCrontab, tools.readCrontab())
-
     def test_splitCommands(self):
         ret = list(tools.splitCommands(['echo foo;'],
                                        head='echo start;',
@@ -744,27 +499,53 @@ class TestTools(generic.TestCase):
             ret[0],
             'echo start;echo foo;echo foo;echo foo;echo end')
 
-    def test_isIPv6Address(self):
-        self.assertTrue(tools.isIPv6Address('fd00:0::5'))
-        self.assertTrue(tools.isIPv6Address('2001:db8:0:8d3:0:8a2e:70:7344'))
-        self.assertFalse(tools.isIPv6Address('foo.bar'))
-        self.assertFalse(tools.isIPv6Address('192.168.1.1'))
-        self.assertFalse(tools.isIPv6Address('fd00'))
 
-    def test_excapeIPv6Address(self):
-        self.assertEqual(tools.escapeIPv6Address('fd00:0::5'), '[fd00:0::5]')
-        self.assertEqual(
-            tools.escapeIPv6Address('2001:db8:0:8d3:0:8a2e:70:7344'),
-            '[2001:db8:0:8d3:0:8a2e:70:7344]')
-        self.assertEqual(tools.escapeIPv6Address('foo.bar'), 'foo.bar')
-        self.assertEqual(tools.escapeIPv6Address('192.168.1.1'), '192.168.1.1')
-        self.assertEqual(tools.escapeIPv6Address('fd00'), 'fd00')
+class TestEscapeIPv6(generic.TestCase):
+    def test_escaped(self):
+        values_and_expected = (
+            ('fd00:0::5', '[fd00:0::5]'),
+            (
+                '2001:db8:0:8d3:0:8a2e:70:7344',
+                '[2001:db8:0:8d3:0:8a2e:70:7344]'
+            ),
+            ('::', '[::]'),
+            ('::1', '[::1]'),
+            ('::ffff:192.0.2.128', '[::ffff:192.0.2.128]'),
+            ('fe80::1', '[fe80::1]'),
+        )
 
-    def test_camelCase(self):
-        self.assertEqual(tools.camelCase('foo'), 'Foo')
-        self.assertEqual(tools.camelCase('Foo'), 'Foo')
-        self.assertEqual(tools.camelCase('foo_bar'), 'FooBar')
-        self.assertEqual(tools.camelCase('foo_Bar'), 'FooBar')
+        for val, exp in values_and_expected:
+            with self.subTest(val=val, exp=exp):
+                self.assertEqual(tools.escapeIPv6Address(val), exp)
+
+    def test_passed(self):
+        test_values = (
+            '192.168.1.1',
+            '172.17.1.133',
+            '255.255.255.255',
+            '169.254.0.1',
+        )
+
+        for val in test_values:
+            with self.subTest(val=val):
+                # IPv4 addresses are not escaped
+                self.assertEqual(tools.escapeIPv6Address(val), val)
+
+    def test_invalid(self):
+        """Invalid IP addresses and hostnames"""
+        test_values = (
+            'foo.bar',
+            'fd00',
+            '2001:0db8:::0000:0000:8a2e:0370:7334',
+            ':2001:0db8:85a3:0000:0000:8a2e:0370:7334',
+            '2001:0gb8:85a3:0000:0000:8a2e:0370:7334',
+            '2001:0db8:85a3:0000:0000:8a2e:0370:7334:abcd',
+            'localhost',
+        )
+
+        for val in test_values:
+            with self.subTest(val=val):
+                self.assertEqual(tools.escapeIPv6Address(val), val)
 
 
 class TestToolsEnviron(generic.TestCase):
@@ -1033,3 +814,43 @@ class TestToolsExecuteOsSystem(generic.TestCase):
     def test_pausable(self):
         proc = tools.Execute('true')
         self.assertFalse(proc.pausable)
+
+
+class Tools_FakeFS(pyfakefs_ut.TestCase):
+    """Tests using a fake filesystem."""
+
+    def setUp(self):
+        self.setUpPyfakefs(allow_root_user=False)
+
+    def test_git_repo_info_none(self):
+        """Actually not a git repo"""
+
+        self.assertEqual(tools.get_git_repository_info(), None)
+
+    def test_git_repo_info(self):
+        """Simulate a git repo"""
+
+        path = pathlib.Path('.git')
+        path.mkdir()
+
+        # Branch folders and hash containing file
+        foobar = path / 'refs' / 'heads' / 'fix' / 'foobar'
+        foobar.parent.mkdir(parents=True)
+
+        with foobar.open('w') as handle:
+            handle.write('01234')
+
+        # HEAD file
+        head = path / 'HEAD'
+
+        with head.open('w') as handle:
+            handle.write('ref: refs/heads/fix/foobar')
+
+        # Test
+        self.assertEqual(
+            tools.get_git_repository_info(),
+            {
+                'hash': '01234',
+                'branch': 'fix/foobar'
+            }
+        )
