@@ -1,20 +1,22 @@
-#    Back In Time
-#    Copyright (C) 2008-2022 Oprea Dan, Bart de Koning, Richard Bailey,
-#    Germar Reitze, Taylor Raack
+"""Collection of helper functions not fitting to other modules.
+"""
+# Back In Time
+# Copyright (C) 2008-2022 Oprea Dan, Bart de Koning, Richard Bailey,
+# Germar Reitze, Taylor Raack
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 2 of the License, or
-#    (at your option) any later version.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
-#    You should have received a copy of the GNU General Public License along
-#    with this program; if not, write to the Free Software Foundation, Inc.,
-#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 import os
 import sys
 import pathlib
@@ -25,18 +27,15 @@ import re
 import errno
 import gzip
 import tempfile
+import locale
 import gettext
-try:
-    from collections.abc import MutableSet
-except ImportError:
-    from collections import MutableSet
+from collections.abc import MutableSet
 import hashlib
 import ipaddress
 import atexit
 from datetime import datetime
 from packaging.version import Version
 from time import sleep
-
 import logger
 
 # Try to import keyring
@@ -87,18 +86,20 @@ DISK_BY_UUID = '/dev/disk/by-uuid'
 # | Handling paths  |
 # |-----------------|
 
-
 def sharePath():
     """Get path where Back In Time is installed.
 
-    If running from source return default ``/usr/share``.
+    This is similar to ``XDG_DATA_DIRS``. If running from source return
+    default ``/usr/share``.
+
+    Share path like: ::
+
+        /usr/share
+        /usr/local/share
+        /opt/usr/share
 
     Returns:
-        str:    share path like::
-
-                    /usr/share
-                    /usr/local/share
-                    /opt/usr/share
+        str: Share path.
     """
     share = os.path.abspath(
         os.path.join(__file__, os.pardir, os.pardir, os.pardir)
@@ -110,6 +111,44 @@ def sharePath():
     return '/usr/share'
 
 
+def backintimePath(*path):
+    """
+    Get path inside ``backintime`` install folder.
+
+    Args:
+        *path (str): Paths that should be joined to ``backintime``.
+
+    Returns:
+        str: Child path of ``backintime`` child path e.g.
+            ``/usr/share/backintime/common``or ``/usr/share/backintime/qt``.
+    """
+    return os.path.abspath(os.path.join(__file__, os.pardir, os.pardir, *path))
+
+
+def docPath():
+    """Not sure what this path is about.
+    """
+    path = pathlib.Path(sharePath()) / 'doc' / 'backintime-common'
+
+    # Dev note (buhtz, aryoda, 2024-02):
+    # This piece of code originally resisted in Config.__init__() and was
+    # introduced by Dan in 2008. The reason for the existence of this "if"
+    # is unclear.
+
+    # Makefile (in common) does only install into share/doc/backintime-common
+    # but never into the the backintime "binary" path so I guess the if is
+    # a) either a distro-specific exception for a distro package that
+    # (manually?) installs the LICENSE into another path
+    # b) or a left-over from old code where the LICENSE was installed
+    # differently...
+
+    license_file = pathlib.Path(backintimePath()) / 'LICENSE'
+    if license_file.exists():
+        path = backintimePath()
+
+    return str(path)
+
+
 # |---------------------------------------------------|
 # | Internationalization (i18n) & localization (L10n) |
 # |---------------------------------------------------|
@@ -193,7 +232,55 @@ def initiate_translation(language_code):
     )
     translation.install(names=['ngettext'])
 
-    return _determine_current_used_language_code(translation, language_code)
+    used_code = _determine_current_used_language_code(
+        translation, language_code)
+
+    set_lc_time_by_language_code(used_code)
+
+    return used_code
+
+
+def set_lc_time_by_language_code(language_code: str):
+    """Set ``LC_TIME`` based on a specific language code.
+
+    Args:
+        language_code(str): A language code consisting of two letters.
+
+    The reason is to display correctly translated weekday and months
+    names. Python's :mod:`datetime` module, as well
+    ``PyQt6.QtCore.QDate``, use :mod:`locale` to determine the
+    correct translation. The module :mod:`gettext` and
+    ``PyQt6.QtCore.QTranslator`` is not involved so their setup does
+    not take effect.
+
+    Be aware that a language code (e.g. ``de``) is not the same as a locale code
+    (e.g. ``de_DE.UTF-8``). This function attempts to determine the latter based
+    on the language code. A warning is logged if it is not possible.
+    """
+
+    # Determine the normalized locale code (e.g. "de_DE.UTF-8") by
+    # language code (e.g. "de").
+
+    # "de" -> "de_DE.ISO8859-1" -> "de_DE"
+    code = locale.normalize(language_code).split('.')[0]
+
+    try:
+        # "de_DE" -> "de_DE.UTF-8"
+        code = code + '.' + locale.getencoding()
+    except AttributeError:  # Python 3.10 or older
+        code = code + '.' + locale.getpreferredencoding()
+
+    try:
+        logger.debug(f'Try to set locale.LC_TIME to "{code}" based on '
+                     f'language code "{language_code}".')
+        locale.setlocale(locale.LC_TIME, code)
+
+    except locale.Error:
+        logger.warning(
+            f'Determined normalized locale code "{code}" (from language code '
+            f'"{code}" not available or invalid. The code will be ignored. '
+            'This might lead to unusual display of dates and timestamps, but '
+            'it does not affect the functionality of the application.')
 
 
 def get_available_language_codes():
@@ -295,22 +382,6 @@ def get_native_language_and_completeness(language_code):
 # |------------------------------------|
 # | Miscellaneous, not categorized yet |
 # |------------------------------------|
-
-def backintimePath(*path):
-    """
-    Get path inside 'backintime' install folder.
-
-    Args:
-        *path (str):    paths that should be joined to 'backintime'
-
-    Returns:
-        str:            'backintime' child path like::
-
-                            /usr/share/backintime/common
-                            /usr/share/backintime/qt
-    """
-    return os.path.abspath(os.path.join(__file__, os.pardir, os.pardir, *path))
-
 def registerBackintimePath(*path):
     """
     Add BackInTime path ``path`` to :py:data:`sys.path` so subsequent imports
@@ -324,18 +395,25 @@ def registerBackintimePath(*path):
         would need this to actually import :py:mod:`tools`.
     """
     path = backintimePath(*path)
-    if not path in sys.path:
+
+    if path not in sys.path:
         sys.path.insert(0, path)
 
+
 def runningFromSource():
-    """
-    Check if BackInTime is running from source (without installing).
+    """Check if BackInTime is running from source (without installing).
+
+    Dev notes by buhtz (2024-04): This function is dangerous and will give a
+    false-negative in fake filesystems (e.g. PyFakeFS). The function should
+    not exist. Beside unit tests it is used only two times. Remove it until
+    migration to pyproject.toml based project packaging (#1575).
 
     Returns:
-        bool:   ``True`` if BackInTime is running from source
+        bool: ``True`` if BackInTime is running from source.
     """
     return os.path.isfile(backintimePath('common', 'backintime'))
 
+
 def addSourceToPathEnviron():
     """
     Add 'backintime/common' path to 'PATH' environ variable.
@@ -343,40 +421,61 @@ def addSourceToPathEnviron():
     source = backintimePath('common')
     path = os.getenv('PATH')
     if path and source not in path.split(':'):
-        os.environ['PATH'] = '%s:%s' %(source, path)
+        os.environ['PATH'] = '%s:%s' % (source, path)
 
-def gitRevisionAndHash():
-    """
-    Get the current Git Branch and the last HashID (shot form) if running
-    from source.
+
+def get_git_repository_info(path=None, hash_length=None):
+    """Return the current branch and last commit hash.
+
+    About the length of a commit hash. There is no strict rule but it is
+    common sense that 8 to 10 characters are enough to be unique.
+
+    Credits: https://stackoverflow.com/a/51224861/4865723
+
+    Args:
+        path (Path): Path with '.git' folder in (default is
+                     current working directory).
+        cut_hash (int): Restrict length of commit hash.
 
     Returns:
-        tuple:  two items of either :py:class:`str` instance if running from
-                source or ``None``
+        (dict): Dict with keys "branch" and "hash" if it is a git repo,
+                otherwise an `None`.
     """
-    ref, hashid = None, None
-    gitPath = os.path.abspath(os.path.join(__file__, os.pardir, os.pardir, '.git'))
-    headPath = os.path.join(gitPath, 'HEAD')
-    refPath = ''
-    if not os.path.isdir(gitPath):
-        return (ref, hashid)
-    try:
-        with open(headPath, 'rt') as f:
-            refPath = f.read().strip('\n')
-            if refPath.startswith('ref: '):
-                refPath = refPath[5:]
-            if refPath:
-                refPath = os.path.join(gitPath, refPath)
-                ref = os.path.basename(refPath)
-    except Exception as e:
-        pass
-    if os.path.isfile(refPath):
-        try:
-            with open(refPath, 'rt') as f:
-                hashid = f.read().strip('\n')[:7]
-        except:
-            pass
-    return (ref, hashid)
+
+    if not path:
+        # Default is current working dir
+        path = pathlib.Path.cwd()
+    elif isinstance(path, str):
+        # WORKAROUND until cmoplete migration to pathlib
+        path = pathlib.Path(path)
+
+    git_folder = path / '.git'
+
+    if not git_folder.exists():
+        return None
+
+    result = {}
+
+    # branch name
+    with (git_folder / 'HEAD').open('r') as handle:
+        val = handle.read()
+
+    if not val.startswith('ref: '):
+        result['branch'] = '(detached HEAD)'
+        result['hash'] = val
+
+    else:
+        result['branch'] = '/'.join(val.split('/')[2:]).strip()
+
+        # commit hash
+        with (git_folder / 'refs' / 'heads' / result['branch']) \
+            .open('r') as handle:
+            result['hash'] = handle.read().strip()
+
+    if hash_length:
+        result['hash'] = result['hash'][:hash_length]
+
+    return result
 
 
 def readFile(path, default=None):
@@ -440,15 +539,15 @@ def readFileLines(path, default = None):
 
     return ret_val
 
+
 def checkCommand(cmd):
-    """
-    Check if command ``cmd`` is a file in 'PATH' environ.
+    """Check if command ``cmd`` is a file in 'PATH' environment.
 
     Args:
-        cmd (str):  command
+        cmd (str): The command.
 
     Returns:
-        bool:       ``True`` if command ``cmd`` is in 'PATH' environ
+        bool: ``True`` if ``cmd`` is in 'PATH' environment otherwise ``False``.
     """
     cmd = cmd.strip()
 
@@ -457,31 +556,42 @@ def checkCommand(cmd):
 
     if os.path.isfile(cmd):
         return True
-    return not which(cmd) is None
+
+    return which(cmd) is not None
+
 
 def which(cmd):
-    """
-    Get the fullpath of executable command ``cmd``. Works like
-    command-line 'which' command.
+    """Get the fullpath of executable command ``cmd``.
+
+    Works like command-line 'which' command.
+
+    Dev note by buhtz (2024-04): Give false-negative results in fake
+    filesystems. Quit often use in the whole code base. But not sure why
+    can we replace it with "which" from shell?
 
     Args:
-        cmd (str):  command
+        cmd (str): The command.
 
     Returns:
-        str:        fullpath of command ``cmd`` or ``None`` if command is
-                    not available
+        str: Fullpath of command ``cmd`` or ``None`` if command is not
+             available.
     """
     pathenv = os.getenv('PATH', '')
-    path = pathenv.split(":")
+    path = pathenv.split(':')
     common = backintimePath('common')
+
     if runningFromSource() and common not in path:
         path.insert(0, common)
+
     for directory in path:
         fullpath = os.path.join(directory, cmd)
+
         if os.path.isfile(fullpath) and os.access(fullpath, os.X_OK):
             return fullpath
+
     return None
 
+
 def makeDirs(path):
     """
     Create directories ``path`` recursive and return success.
@@ -498,15 +608,19 @@ def makeDirs(path):
 
     if os.path.isdir(path):
         return True
+
     else:
+
         try:
             os.makedirs(path)
         except Exception as e:
             logger.error("Failed to make dirs '%s': %s"
-                         %(path, str(e)), traceDepth = 1)
+                         % (path, str(e)), traceDepth=1)
+
     return os.path.isdir(path)
 
-def mkdir(path, mode = 0o755, enforce_permissions = True):
+
+def mkdir(path, mode=0o755, enforce_permissions=True):
     """
     Create directory ``path``.
 
@@ -523,14 +637,18 @@ def mkdir(path, mode = 0o755, enforce_permissions = True):
                 os.chmod(path, mode)
         except:
             return False
+
         return True
+
     else:
         os.mkdir(path, mode)
+
         if mode & 0o002 == 0o002:
-            #make file world (other) writable was requested
-            #debian and ubuntu won't set o+w with os.mkdir
-            #this will fix it
+            # make file world (other) writable was requested
+            # debian and ubuntu won't set o+w with os.mkdir
+            # this will fix it
             os.chmod(path, mode)
+
     return os.path.isdir(path)
 
 
@@ -675,7 +793,7 @@ def checkXServer():
     """
     Check if there is a X11 server running on this system.
 
-    Use ``is_Qt5_working`` instead if you want to be sure that Qt5 is working.
+    Use ``is_Qt_working`` instead if you want to be sure that Qt is working.
 
     Returns:
         bool:   ``True`` if X11 server is running
@@ -696,22 +814,22 @@ def checkXServer():
         return False
 
 
-def is_Qt5_working(systray_required=False):
+def is_Qt_working(systray_required=False):
     """
-    Check if the Qt5 GUI library is working (installed and configured)
+    Check if the Qt GUI library is working (installed and configured)
 
-    This function is contained in BiT CLI (not BiT Qt) to allow Qt5
+    This function is contained in BiT CLI (not BiT Qt) to allow Qt
     diagnostics output even if the BiT Qt GUI is not installed.
-    This function does NOT add a hard Qt5 dependency (just "probing")
+    This function does NOT add a hard Qt dependency (just "probing")
     so it is OK to be in BiT CLI.
 
     Args:
         systray_required: Set to ``True`` if the systray of the desktop
-        environment must be available too to consider Qt5 as "working"
+        environment must be available too to consider Qt as "working"
 
     Returns:
-        bool: ``True``  Qt5 can create a GUI
-              ``False`` Qt5 fails (or the systray is not available
+        bool: ``True``  Qt can create a GUI
+              ``False`` Qt fails (or the systray is not available
                         if ``systray_required`` is ``True``)
     """
 
@@ -719,7 +837,7 @@ def is_Qt5_working(systray_required=False):
     # don't want to crash BiT if this happens...
 
     try:
-        path = os.path.join(backintimePath("common"), "qt5_probing.py")
+        path = os.path.join(backintimePath("common"), "qt_probing.py")
         cmd = [sys.executable, path]
         if logger.DEBUG:
             cmd.append('--debug')
@@ -730,27 +848,27 @@ def is_Qt5_working(systray_required=False):
                               universal_newlines=True) as proc:
 
             std_output, error_output = proc.communicate(timeout=30)  # to get the exit code
-            # "timeout" fixes #1592 (qt5_probing.py may hang as root): Kill after timeout
+            # "timeout" fixes #1592 (qt_probing.py may hang as root): Kill after timeout
 
-            logger.debug(f"Qt5 probing result: exit code {proc.returncode}")
+            logger.debug(f"Qt probing result: exit code {proc.returncode}")
 
-            if proc.returncode != 2 or logger.DEBUG:  # if some Qt5 parts are missing: Show details
-                logger.debug(f"Qt5 probing stdout:\n{std_output}")
-                logger.debug(f"Qt5 probing errout:\n{error_output}")
+            if proc.returncode != 2 or logger.DEBUG:  # if some Qt parts are missing: Show details
+                logger.debug(f"Qt probing stdout:\n{std_output}")
+                logger.debug(f"Qt probing errout:\n{error_output}")
 
             return proc.returncode == 2 or (proc.returncode == 1 and systray_required is False)
 
     except FileNotFoundError:
-        logger.error(f"Qt5 probing script not found: {cmd[0]}")
+        logger.error(f"Qt probing script not found: {cmd[0]}")
         raise
 
-    # Fix for #1592 (qt5_probing.py may hang as root): Kill after timeout
+    # Fix for #1592 (qt_probing.py may hang as root): Kill after timeout
     except subprocess.TimeoutExpired:
         proc.kill()
         outs, errs = proc.communicate()
-        logger.info("Qt5 probing sub process killed after timeout without response")
-        logger.debug(f"Qt5 probing stdout:\n{outs}")
-        logger.debug(f"Qt5 probing errout:\n{errs}")
+        logger.info("Qt probing sub process killed after timeout without response")
+        logger.debug(f"Qt probing stdout:\n{outs}")
+        logger.debug(f"Qt probing errout:\n{errs}")
 
     except Exception as e:
         logger.error(f"Error: {repr(e)}")
@@ -904,6 +1022,9 @@ def rsyncPrefix(config,
     else:
         cmd.append('--links')
 
+    if config.oneFileSystem():
+        cmd.append('--one-file-system')
+
     if config.preserveAcl() and "ACLs" in caps:
         cmd.append('--acls')  # preserve ACLs (implies --perms)
         no_perms = False
@@ -1040,6 +1161,9 @@ def checkCronPattern(s):
 
     Returns:
         bool:       ``True`` if ``s`` is a valid cron pattern
+
+    Dev note: Schedule for removal. See comment in
+    `config.Config.saveProfile()`.
     """
     if s.find(' ') >= 0:
         return False
@@ -1058,6 +1182,7 @@ def checkCronPattern(s):
     except ValueError:
         return False
 
+
 #TODO: check if this is still necessary
 def checkHomeEncrypt():
     """
@@ -1084,6 +1209,7 @@ def checkHomeEncrypt():
                 return True
     return False
 
+
 def envLoad(f):
     """
     Load environ variables from file ``f`` into current environ.
@@ -1101,7 +1227,8 @@ def envLoad(f):
             continue
         if not key in list(env.keys()):
             os.environ[key] = value
-    del(env_file)
+    del env_file
+
 
 def envSave(f):
     """
@@ -1122,6 +1249,7 @@ def envSave(f):
 
     env_file.save(f)
 
+
 def keyringSupported():
     """
     Checks if a keyring (supported by BiT) is available
@@ -1228,12 +1356,14 @@ def password(*args):
         return keyring.get_password(*args)
     return None
 
+
 def setPassword(*args):
 
     if is_keyring_available:
         return keyring.set_password(*args)
     return False
 
+
 def mountpoint(path):
     """
     Get the mountpoint of ``path``. If your HOME is on a separate partition
@@ -1246,12 +1376,16 @@ def mountpoint(path):
         str:        mountpoint of the filesystem
     """
     path = os.path.realpath(os.path.abspath(path))
+
     while path != os.path.sep:
         if os.path.ismount(path):
             return path
+
         path = os.path.abspath(os.path.join(path, os.pardir))
+
     return path
 
+
 def decodeOctalEscape(s):
     """
     Decode octal-escaped characters with its ASCII dependence.
@@ -1267,6 +1401,7 @@ def decodeOctalEscape(s):
         return chr(int(m.group(1), 8))
     return re.sub(r'\\(\d{3})', repl, s)
 
+
 def mountArgs(path):
     """
     Get all /etc/mtab args for the filesystem of ``path`` as a list.
@@ -1283,7 +1418,7 @@ def mountArgs(path):
         list:       mount args
     """
     mp = mountpoint(path)
-    
+
     with open('/etc/mtab', 'r') as mounts:
 
         for line in mounts:
@@ -1297,6 +1432,7 @@ def mountArgs(path):
 
     return None
 
+
 def device(path):
     """
     Get the device for the filesystem of ``path``.
@@ -1319,6 +1455,7 @@ def device(path):
 
     return None
 
+
 def filesystem(path):
     """
     Get the filesystem type for the filesystem of ``path``.
@@ -1461,6 +1598,7 @@ def uuidFromDev(dev):
     # Try "udevadm" command at the end
     return _uuidFromDev_via_udevadm_command(dev)
 
+
 def uuidFromPath(path):
     """
     Get the UUID for the for the filesystem of ``path``.
@@ -1473,31 +1611,6 @@ def uuidFromPath(path):
     """
     return uuidFromDev(device(path))
 
-def filesystemMountInfo():
-    """
-    Get a dict of mount point string -> dict of filesystem info for
-    entire system.
-
-    Returns:
-        dict:   {MOUNTPOINT: {'original_uuid': UUID}}
-    """
-    # There may be multiple mount points inside of the root (/) mount, so
-    # iterate over mtab to find all non-special mounts.
-    with open('/etc/mtab', 'r') as mounts:
-        return {items[1]: {'original_uuid': uuidFromDev(items[0])} for items in
-                [mount_line.strip('\n').split(' ')[:2] for mount_line in mounts]
-                if uuidFromDev(items[0]) != None}
-
-
-def syncfs():
-    """
-    Sync any data buffered in memory to disk.
-
-    Returns:
-        bool:   ``True`` if successful
-    """
-    if checkCommand('sync'):
-        return(Execute(['sync']).run() == 0)
 
 def isRoot():
     """
@@ -1545,50 +1658,61 @@ def patternHasNotEncryptableWildcard(pattern):
         return True
     return False
 
-BIT_TIME_FORMAT = '%Y%m%d %H%M'
-ANACRON_TIME_FORMAT = '%Y%m%d'
 
 def readTimeStamp(fname):
     """
     Read date string from file ``fname`` and try to return datetime.
 
     Args:
-        fname (str):        full path to timestamp file
+        fname (str): Full path to timestamp file.
 
     Returns:
-        datetime.datetime:  date from timestamp file
+        datetime.datetime: Timestamp object.
     """
+
     if not os.path.exists(fname):
-        logger.debug("no timestamp in '%(file)s'" %
-                     {'file': fname})
+        logger.debug(f"No timestamp file '{fname}'")
         return
+
     with open(fname, 'r') as f:
         s = f.read().strip('\n')
-    for i in (ANACRON_TIME_FORMAT, BIT_TIME_FORMAT):
+
+    time_formats = (
+        '%Y%m%d %H%M',  # BIT like
+        '%Y%m%d',  # Anacron like
+    )
+
+    for form in time_formats:
+
         try:
-            stamp = datetime.strptime(s, i)
-            logger.debug("read timestamp '%(time)s' from file '%(file)s'" %
-                         {'time': stamp,
-                          'file': fname})
-            return stamp
+            stamp = datetime.strptime(s, form)
+
         except ValueError:
+            # invalid format
+            # next iteration
             pass
 
+        else:
+            # valid time stamp
+            logger.debug(f"Read timestamp '{stamp}' from file '{fname}'")
+
+            return stamp
+
+
 def writeTimeStamp(fname):
-    """
-    Write current date and time into file ``fname``.
+    """Write current date and time into file ``fname``.
 
     Args:
-        fname (str):        full path to timestamp file
+        fname (str): Full path to timestamp file.
     """
-    now = datetime.now().strftime(BIT_TIME_FORMAT)
-    logger.debug("write timestamp '%(time)s' into file '%(file)s'" %
-                 {'time': now,
-                  'file': fname})
+    now = datetime.now().strftime('%Y%m%d %H%M')
+    logger.debug(f"Write timestamp '{now}' into file '{fname}'")
     makeDirs(os.path.dirname(fname))
+
     with open(fname, 'w') as f:
         f.write(now)
 
+
 INHIBIT_LOGGING_OUT = 1
 INHIBIT_USER_SWITCHING = 2
 INHIBIT_SUSPENDING = 4
@@ -1680,71 +1804,6 @@ def unInhibitSuspend(cookie, bus, dbus_props):
         logger.warning('Release inhibit Suspend failed.')
         return (cookie, bus, dbus_props)
 
-def readCrontab():
-    """
-    Read users crontab.
-
-    Returns:
-        list:   crontab lines
-    """
-    cmd = ['crontab', '-l']
-    if not checkCommand(cmd[0]):
-        logger.debug('crontab not found.')
-        return []
-    else:
-        proc = subprocess.Popen(cmd,
-                                stdout = subprocess.PIPE,
-                                stderr = subprocess.PIPE,
-                                universal_newlines = True)
-        out, err = proc.communicate()
-        if proc.returncode or err:
-            logger.error('Failed to get crontab lines: %s, %s'
-                         %(proc.returncode, err))
-            return []
-        else:
-            crontab = [x.strip() for x in out.strip('\n').split('\n')]
-            if crontab == ['']:  # Fixes issue #1181 (line count of empty crontab was 1 instead of 0)
-                crontab = []
-            logger.debug('Read %s lines from user crontab'
-                         %len(crontab))
-            return crontab
-
-def writeCrontab(lines):
-    """
-    Write to users crontab.
-
-    Note:
-        This will overwrite the whole crontab. So to keep the old crontab and
-        only add new entries you need to read it first with
-        :py:func:`tools.readCrontab`, append new entries to the list and write
-        it back.
-
-    Args:
-        lines (:py:class:`list`, :py:class:`tuple`):
-                    lines that should be written to crontab
-
-    Returns:
-        bool:       ``True`` if successful
-    """
-    assert isinstance(lines, (list, tuple)), 'lines is not list or tuple type: %s' % lines
-    with tempfile.NamedTemporaryFile(mode = 'wt') as f:
-        f.write('\n'.join(lines))
-        f.write('\n\n')
-        f.flush()
-        cmd = ['crontab', f.name]
-        proc = subprocess.Popen(cmd,
-                                stdout = subprocess.DEVNULL,
-                                stderr = subprocess.PIPE,
-                                universal_newlines = True)
-        out, err = proc.communicate()
-    if proc.returncode or err:
-        logger.error('Failed to write lines to crontab: %s, %s'
-                     %(proc.returncode, err))
-        return False
-    else:
-        logger.debug('Wrote %s lines to user crontab'
-                     %len(lines))
-        return True
 
 def splitCommands(cmds, head = '', tail = '', maxLength = 0):
     """
@@ -1774,36 +1833,30 @@ def splitCommands(cmds, head = '', tail = '', maxLength = 0):
         s += tail
         yield s
 
-def isIPv6Address(address):
-    """
-    Check if ``address`` is a valid IPv6 address.
-
-    Args:
-        address (str):  address that should get tested
-
-    Returns:
-        bool:           True if ``address`` is a valid IPv6 address
-    """
-    try:
-        return isinstance(ipaddress.IPv6Address(address), ipaddress.IPv6Address)
-    except:
-        return False
 
 def escapeIPv6Address(address):
-    """
-    Escape IPv6 Addresses with square brackets ``[]``.
+    """Escape IP addresses with square brackets ``[]`` if they are IPv6.
+
+    If it is an IPv4 address or a hostname (lettersonly) nothing is changed.
 
     Args:
-        address (str):  address that should be escaped
+        address (str): IP-Address to escape if needed.
 
     Returns:
-        str:            ``address`` in square brackets
+        str: The address, escaped if it is IPv6.
     """
-    if isIPv6Address(address):
-        return '[{}]'.format(address)
-    else:
+    try:
+        ip = ipaddress.ip_address(address)
+    except ValueError:
+        # invalid IP, e.g. a hostname
         return address
 
+    if ip.version == 6:
+        return f'[{address}]'
+
+    return address
+
+
 def camelCase(s):
     """
     Remove underlines and make every first char uppercase.
@@ -1832,6 +1885,7 @@ def fdDup(old, new_fd, mode = 'w'):
     except OSError as e:
         logger.debug('Failed to redirect {}: {}'.format(old, str(e)))
 
+
 class UniquenessSet:
     """
     Check for uniqueness or equality of files.
@@ -1958,20 +2012,17 @@ class Alarm(object):
     (reentrance) or you may cause non-deterministic "random" RTEs.
     """
     def __init__(self, callback = None, overwrite = True):
-        """
-        Create a new alarm instance
+        """Create a new alarm instance
 
         Args:
-            callback: Function to call when the timer ran down
-                      (ensure calling only reentrant code).
-                      Use ``None`` to throw a ``Timeout`` exception instead.
-            overwrite: Is it allowed to (re)start the timer
-                       even though the current timer is still running
-                       ("ticking"):
-                       ``True`` cancels the current timer (if active)
-                                and restarts with the new timeout.
-                       ``False` silently ignores the start request
-                                if the current timer is still "ticking"
+            callback: Function to call when the timer ran down (ensure
+                calling only reentrant code). Use ``None`` to throw a
+                ``Timeout`` exception instead.
+            overwrite: Is it allowed to (re)start the timer even though the
+                current timer is still running ("ticking"). ``True`` cancels
+                the current timer (if active) and restarts with the new
+                timeout. ``False`` silently ignores the start request if the
+                current timer is still "ticking"
         """
         self.callback = callback
         self.ticking = False
@@ -2024,6 +2075,7 @@ class Alarm(object):
         else:
             self.callback()
 
+
 class ShutDown(object):
     """
     Shutdown the system after the current snapshot has finished.
@@ -2142,7 +2194,7 @@ class ShutDown(object):
                 sessionbus = dbus.SessionBus()
             systembus  = dbus.SystemBus()
         except:
-            return((None, None))
+            return (None, None)
         des = list(self.DBUS_SHUTDOWN.keys())
         des.sort()
         for de in des:
@@ -2156,23 +2208,23 @@ class ShutDown(object):
                     bus = systembus
                 interface = bus.get_object(dbus_props['service'], dbus_props['objectPath'])
                 proxy = interface.get_dbus_method(dbus_props['method'], dbus_props['interface'])
-                return((proxy, dbus_props['arguments']))
+                return (proxy, dbus_props['arguments'])
             except dbus.exceptions.DBusException:
                 continue
-        return((None, None))
+        return (None, None)
 
     def canShutdown(self):
         """
         Indicate if a valid dbus service is available to shutdown system.
         """
-        return(not self.proxy is None or self.is_root)
+        return not self.proxy is None or self.is_root
 
     def askBeforeQuit(self):
         """
         Indicate if ShutDown is ready to fire and so the application
         shouldn't be closed.
         """
-        return(self.activate_shutdown and not self.started)
+        return self.activate_shutdown and not self.started
 
     def shutdown(self):
         """
@@ -2180,19 +2232,21 @@ class ShutDown(object):
         call the dbus proxy to start the shutdown.
         """
         if not self.activate_shutdown:
-            return(False)
+            return False
+
         if self.is_root:
-            syncfs()
             self.started = True
             proc = subprocess.Popen(['shutdown', '-h', 'now'])
             proc.communicate()
             return proc.returncode
+
         if self.proxy is None:
-            return(False)
+            return False
+
         else:
-            syncfs()
             self.started = True
-            return(self.proxy(*self.args))
+
+            return self.proxy(*self.args)
 
     def unity7(self):
         """
@@ -2209,6 +2263,7 @@ class ShutDown(object):
 
         return m and Version(m.group(1)) >= Version('7.0') and processExists('unity-panel-service')
 
+
 class SetupUdev(object):
     """
     Setup Udev rules for starting BackInTime when a drive get connected.
@@ -2219,14 +2274,18 @@ class SetupUdev(object):
     OBJECT = '/UdevRules'
     INTERFACE = 'net.launchpad.backintime.serviceHelper.UdevRules'
     MEMBERS = ('addRule', 'save', 'delete')
+
     def __init__(self):
         if dbus is None:
             self.isReady = False
+
             return
+
         try:
             bus = dbus.SystemBus()
             conn = bus.get_object(SetupUdev.CONNECTION, SetupUdev.OBJECT)
             self.iface = dbus.Interface(conn, SetupUdev.INTERFACE)
+
         except dbus.exceptions.DBusException as e:
             # Only DBusExceptions are  handled to do a "graceful recovery"
             # by working without a serviceHelper D-Bus connection...
@@ -2235,55 +2294,68 @@ class SetupUdev(object):
             # if e._dbus_error_name in ('org.freedesktop.DBus.Error.NameHasNoOwner',
             #                           'org.freedesktop.DBus.Error.ServiceUnknown',
             #                           'org.freedesktop.DBus.Error.FileNotFound'):
-            logger.warning("Failed to connect to Udev serviceHelper daemon via D-Bus: " + e.get_dbus_name())
-            logger.warning("D-Bus message: " + e.get_dbus_message())
-            logger.warning("Udev-based profiles cannot be changed or checked due to Udev serviceHelper connection failure")
+            logger.warning('Failed to connect to Udev serviceHelper daemon '
+                           'via D-Bus: ' + e.get_dbus_name())
+            logger.warning('D-Bus message: ' + e.get_dbus_message())
+            logger.warning('Udev-based profiles cannot be changed or checked '
+                           'due to Udev serviceHelper connection failure')
             conn = None
+
             # else:
             #     raise
+
         self.isReady = bool(conn)
 
     def addRule(self, cmd, uuid):
-        """
-        Prepare rules in serviceHelper.py
+        """Prepare rules in serviceHelper.py
         """
         if not self.isReady:
             return
+
         try:
             return self.iface.addRule(cmd, uuid)
-        except dbus.exceptions.DBusException as e:
-            if e._dbus_error_name == 'net.launchpad.backintime.InvalidChar':
-                raise InvalidChar(str(e))
-            elif e._dbus_error_name == 'net.launchpad.backintime.InvalidCmd':
-                raise InvalidCmd(str(e))
-            elif e._dbus_error_name == 'net.launchpad.backintime.LimitExceeded':
-                raise LimitExceeded(str(e))
+
+        except dbus.exceptions.DBusException as exc:
+            if exc._dbus_error_name == 'net.launchpad.backintime.InvalidChar':
+                raise InvalidChar(str(exc)) from exc
+
+            elif exc._dbus_error_name == 'net.launchpad.backintime.InvalidCmd':
+                raise InvalidCmd(str(exc)) from exc
+
+            elif exc._dbus_error_name == 'net.launchpad.backintime.LimitExceeded':
+                raise LimitExceeded(str(exc))  from exc
+
             else:
                 raise
 
     def save(self):
-        """
-        Save rules with serviceHelper.py after authentication
+        """Save rules with serviceHelper.py after authentication.
+
         If no rules where added before this will delete current rule.
         """
         if not self.isReady:
             return
+
         try:
             return self.iface.save()
-        except dbus.exceptions.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.DeviceDriver.PermissionDeniedByPolicy':
-                raise PermissionDeniedByPolicy(str(e))
+
+        except dbus.exceptions.DBusException as err:
+
+            if err._dbus_error_name == 'com.ubuntu.DeviceDriver.PermissionDeniedByPolicy':
+                raise PermissionDeniedByPolicy(str(err)) from err
+
             else:
-                raise
+                raise err
 
     def clean(self):
-        """
-        Clean up remote cache
+        """Clean up remote cache.
         """
         if not self.isReady:
             return
+
         self.iface.clean()
 
+
 class PathHistory(object):
     def __init__(self, path):
         self.history = [path,]
@@ -2318,6 +2390,7 @@ class PathHistory(object):
         self.history = [path,]
         self.index = 0
 
+
 class OrderedSet(MutableSet):
     """
     OrderedSet from Python recipe
@@ -2379,6 +2452,7 @@ class OrderedSet(MutableSet):
             return len(self) == len(other) and list(self) == list(other)
         return set(self) == set(other)
 
+
 class Execute(object):
     """
     Execute external commands and handle its output.
@@ -2584,6 +2658,7 @@ class Execute(object):
             logger.info('Kill process "%s"' %self.printable_cmd, self.parent, 2)
             return self.currentProc.kill()
 
+
 class Daemon:
     """
     A generic daemon class.
